package cz.cr.sara.kytara.ciselniky;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.domain.TypAkordu;
import cz.cr.sara.kytara.service.AkordManager;

/**
 * Vyber akordu.
 * @author lukas
 */
public class AkordVyber {
  private final VyberAkordu vyberAkordu; // Vyber akordu

  private final static Logger log = LoggerFactory.getLogger(AkordVyber.class);

  private static volatile AkordVyber INSTANCE = null;

  /**
   * Vyber akordu.
   * @param akordManager - manazer pro akordy.
   * @throws NullPointerException
   * @return Vrati vyber akordu.
   */
  public static AkordVyber getInstance(AkordManager akordManager) throws NullPointerException {
    if (INSTANCE == null) {
      synchronized (AkordVyber.class) {
        if (INSTANCE == null) {
          INSTANCE = new AkordVyber(akordManager);
        }
      }
    }
    return INSTANCE;
  }

  /**
   * Vyber akordu.
   * @param akordManager - manazer pro akordy.
   * @throws NullPointerException
   */
  private AkordVyber(AkordManager akordManager) throws NullPointerException {
    try {
      if (akordManager == null) {
        throw new NullPointerException("Chybne vstupni parametry");
      }

      if (log.isDebugEnabled()) {
        log.debug("Vyhledani vyberu akordu");
      }

      // Seznam nazvu tonu
      List<String> nazvyTonu = akordManager.getAllNazev();
      if (nazvyTonu == null || nazvyTonu.size() == 0) {
        throw new NullPointerException("Neexistuje seznam nazvu tonu");
      }

      // Seznam typu akordu
      List<TypAkordu> typyAkordu = akordManager.getAllTypAkordu();
      if (typyAkordu == null || typyAkordu.size() == 0) {
        throw new NullPointerException("Neexistuje seznam typu akordu");
      }
      
      // Seznam vsech tonu
      List<Ton> tony = akordManager.getAllTony();
      if (tony == null || tony.size() == 0) {
        throw new NullPointerException("Neexistuje seznam tonu");
      }

      this.vyberAkordu = new VyberAkordu(nazvyTonu,
          typyAkordu, tony);
    }
    catch (Exception e) {
      throw new NullPointerException("AkordVyber.AkordVyber: " + e.getMessage());
    }
  }

  /**
   * @return Vrati vyber akordu.
   */
  public VyberAkordu getVyberAkordu() {
    return vyberAkordu;
  }

  /**
   * Vyber akordu.
   */
  @Override
  public String toString() {
    return (vyberAkordu != null)
        ? vyberAkordu.toString()
        : "null";
  }
}
