package cz.cr.sara.kytara.ciselniky;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cz.cr.sara.kytara.service.TonManager;
import cz.cr.sara.kytara.service.pojos.AllTony;

/**
 * Vyber tonu.
 * @author lukas
 */
public class TonVyber {
  private final AllTony allTony; // nazvy tonu

  private final static Logger log = LoggerFactory.getLogger(TonVyber.class);

  private static volatile TonVyber INSTANCE = null;

  /**
   * Vyber tonu.
   * @param tonManager - manazer pro tony.
   * @throws NullPointerException
   * @return Vrati vyber tonu.
   */
  public static TonVyber getInstance(TonManager tonManager) throws NullPointerException {
    if (INSTANCE == null) {
      synchronized (TonVyber.class) {
        if (INSTANCE == null) {
          INSTANCE = new TonVyber(tonManager);
        }
      }
    }
    return INSTANCE;
  }

  /**
   * Vyber tonu.
   * @param tonManager - manazer pro tony.
   * @throws NullPointerException
   */
  private TonVyber(TonManager tonManager) throws NullPointerException {
    try {
      if (tonManager == null) {
        throw new NullPointerException("Chybne vstupni parametry");
      }

      if (log.isInfoEnabled()) {
        log.info("Vyhledani vyberu tonu");
      }

      // Seznam nazvu tonu
      List<String> nazvyTonu = tonManager.getAllNazev();
      if (nazvyTonu == null || nazvyTonu.size() == 0) {
        throw new NullPointerException("Neexistuje seznam nazvu tonu");
      }
      this.allTony = new AllTony(nazvyTonu);
    }
    catch (Exception e) {
      throw new NullPointerException("TonVyber.TonVyber: " + e.getMessage());
    }
  }

  /**
   * @return Vrati nazvy tonu.
   */
  public AllTony getAllTony() {
    return allTony;
  }

  @Override
  public String toString() {
    return (allTony != null)
        ? allTony.toString()
        : "null";
  }
}
