package cz.cr.sara.kytara.ciselniky;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.domain.TypAkordu;
import cz.cr.sara.kytara.service.AkordManager;

/**
 * Vyber triad.
 * @author lukas
 */
public class TriadyVyber {
  private final VyberAkordu vyberAkordu; // Vyber akordu

  private final static Logger log = LoggerFactory.getLogger(TriadyVyber.class);

  private static volatile TriadyVyber INSTANCE = null;

  /**
   * Vyber triad.
   * @param akordManager - manazer pro akordy.
   * @throws NullPointerException
   * @return Vrati vyber akordu.
   */
  public static TriadyVyber getInstance(AkordManager akordManager) throws NullPointerException {
    if (INSTANCE == null) {
      synchronized (TriadyVyber.class) {
        if (INSTANCE == null) {
          INSTANCE = new TriadyVyber(akordManager);
        }
      }
    }
    return INSTANCE;
  }

  /**
   * Vyber triad.
   * @param akordManager - manazer pro akordy.
   * @throws NullPointerException
   */
  private TriadyVyber(AkordManager akordManager) throws NullPointerException {
    try {
      if (akordManager == null) {
        throw new NullPointerException("Chybne vstupni parametry");
      }

      if (log.isInfoEnabled()) {
        log.info("Vyhledani vyberu triad");
      }

      // Seznam nazvu tonu
      List<String> nazvyTonu = akordManager.getAllNazev();
      if (nazvyTonu == null || nazvyTonu.size() == 0) {
        throw new NullPointerException("Neexistuje seznam nazvu tonu");
      }

      // Seznam typu akordu
      List<TypAkordu> typyAkordu = akordManager.getAllTriady();
      if (typyAkordu == null || typyAkordu.size() == 0) {
        throw new NullPointerException("Neexistuje seznam typu triad");
      }

      // Seznam vsech tonu
      List<Ton> tony = akordManager.getAllTony();
      if (tony == null || tony.size() == 0) {
        throw new NullPointerException("Neexistuje seznam tonu");
      }
      
      this.vyberAkordu = new VyberAkordu(nazvyTonu,
          typyAkordu, tony);
    }
    catch (Exception e) {
      throw new NullPointerException("TriadyVyber.TriadyVyber: " + e.getMessage());
    }
  }

  /**
   * @return Vrati vyber akordu.
   */
  public VyberAkordu getVyberAkordu() {
    return vyberAkordu;
  }

  /**
   * Vyber triad.
   */
  @Override
  public String toString() {
    return (vyberAkordu != null)
        ? vyberAkordu.toString()
        : "null";
  }
}
