package cz.cr.sara.kytara.ciselniky;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.domain.TypAkordu;

/**
 * Vyber akordu.
 * @author lukas
 */
public class VyberAkordu {
  /** Pocet tonu v jednom radku */
  public static final int COUNT_TONY_LINE = 3;

  private final List<List<String>> nazvyTonu; // Seznam nazvu tonu
  private final Map<String, VyberAkordu.Kategorie> mapKategorie; // Seznam kategorii akordu
  private final List<Ton> tony; //Seznam vsech tonu.

  /**
   * Vyber akordu.
   * @param nazvyTonu - seznam nazvu tonu.
   * @param typyAkordu - seznam typu akordu.
   * @throws NullPointerException
   */
  public VyberAkordu(List<String> nazvyTonu,
      List<TypAkordu> typyAkordu,
      List<Ton> tony) throws NullPointerException {
    try {
      if (nazvyTonu == null || nazvyTonu.size() == 0 || typyAkordu == null || typyAkordu.size() == 0) {
        throw new NullPointerException("Chybne vstupni parametry");
      }

      //Seznam tonu.
      this.tony = tony;

      // Seznam nazvu tonu
      this.nazvyTonu = new ArrayList<List<String>>();
      List<String> lineTony = new ArrayList<String>(VyberAkordu.COUNT_TONY_LINE);
      for (String ton : nazvyTonu) {
        lineTony.add(ton);
        if (lineTony.size() == VyberAkordu.COUNT_TONY_LINE) {
          this.nazvyTonu.add(lineTony);
          lineTony = new ArrayList<String>(VyberAkordu.COUNT_TONY_LINE);
        }
      }
      if (lineTony.size() > 0) {
        this.nazvyTonu.add(lineTony);
      }

      // Seznam kategorii akordu
      this.mapKategorie = new TreeMap<>();
      for (TypAkordu typAkordu : typyAkordu) {
        // Kategorie akordu
        String kategorie = typAkordu.getKategorie();
        VyberAkordu.Kategorie saveKategorie = this.mapKategorie.get(kategorie);
        if (saveKategorie == null) {
          saveKategorie = new VyberAkordu.Kategorie(kategorie);
          this.mapKategorie.put(kategorie,
              saveKategorie);
        }

        // Typ akordu
        saveKategorie.typy.add(new VyberAkordu.Typ(typAkordu));
      }
    }
    catch (Exception e) {
      throw new NullPointerException("VyberAkordu.VyberAkordu: " + e.getMessage());
    }
  }

  /**
   * @return Vrati seznam nazvu tonu.
   */
  public List<List<String>> getNazvyTonu() {
    return nazvyTonu;
  }

  /**
   * @return Vrati seznamu tonu.
   */
  public List<Ton> getTony() {
    return tony;
  }

  /**
   * @return Vrati seznam kategorii akordu.
   */
  public Collection<VyberAkordu.Kategorie> getKategorie() {
    return mapKategorie.values();
  }

  /**
   * Vyber akordu.
   */
  @Override
  public String toString() {
    StringBuffer info = new StringBuffer(100);
    info.append("[NAZVY-TONU=");
    info.append(nazvyTonu);
    info.append("[Tony=");
    info.append(tony);
    info.append(",KATEGORIE-AKORDU=");
    info.append(getKategorie());
    info.append("]");
    return info.toString();
  }

  /**
   * Kategorie akordu.
   * @author lukas
   */
  public static class Kategorie {
    private final String kategorie; // Kategorie akordu
    private final List<VyberAkordu.Typ> typy; // Seznam typu akordu

    /**
     * Kategorie akordu.
     * @param kategorie - kategorie akordu.
     */
    private Kategorie(String kategorie) {
      this.kategorie = kategorie;
      this.typy = new ArrayList<>();
    }

    /**
     * @return Vrati kategorii akordu.
     */
    public String getKategorie() {
      return kategorie;
    }

    /**
     * @return Vrati seznam typu akordu.
     */
    public List<VyberAkordu.Typ> getTypy() {
      return typy;
    }

    /**
     * Kategorie akordu.
     */
    @Override
    public String toString() {
      StringBuffer info = new StringBuffer(50);
      info.append("[KATEGORIE='");
      info.append(kategorie);
      info.append("',POCET-TYPU=");
      info.append((typy != null)
          ? typy.size()
          : 0);
      info.append("]");
      return info.toString();
    }
  }

  /**
   * Typ akordu.
   * @author lukas
   */
  public static class Typ {
    private final String typ; // Typ akordu
    private final String jmeno; // Jmeno akordu

    /**
     * Typ akordu.
     * @param typAkordu - typ akordu.
     */
    private Typ(TypAkordu typAkordu) {
      this.typ = typAkordu.getTyp();
      this.jmeno = typAkordu.getJmeno();
    }

    /**
     * @return Vrati typ akordu.
     */
    public String getTyp() {
      return typ;
    }

    /**
     * @return Vrati jmeno akordu.
     */
    public String getJmeno() {
      return jmeno;
    }

    /**
     * Typ akordu.
     */
    @Override
    public String toString() {
      StringBuffer info = new StringBuffer(50);
      info.append("[TYP='");
      info.append(typ);
      info.append("',JMENO='");
      info.append(jmeno);
      info.append("']");
      return info.toString();
    }
  }
}
