package cz.cr.sara.kytara.domain;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;

import org.hibernate.annotations.GenericGenerator;

/**
 * Informace o intervalu.
 * @author lukas
 */
@Entity
@Table(name = "interval")
@NamedQueries({ @NamedQuery(name = "Interval.findByTypAkordu",
    query = "SELECT i.interval FROM Interval i WHERE i.typAkordu IN (select id FROM TypAkordu where typ = :typ)") })
public class Interval
    implements Serializable {

  private static final long serialVersionUID = 1L;

  private String id; // ID intervalu
  private int interval; // Interval
  private TypAkordu typAkordu; // Typ akordu

  /**
   * @return Vrati ID intervalu.
   */
  @Id
  @GeneratedValue(generator = "system-uuid")
  @GenericGenerator(name = "system-uuid", strategy = "uuid2")
  @Column(name = "id", nullable = false)
  public String getId() {
    return id;
  }

  /**
   * Ulozi ID intervalu.
   * @param id - ID intervalu.
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * @return VVrati interval.
   */
  @Column(name = "interval", nullable = false)
  public int getInterval() {
    return interval;
  }

  /**
   * Ulozi interval.
   * @param interval - interval.
   */
  public void setInterval(int interval) {
    this.interval = interval;
  }

  /**
   * @return Vrati typ akordu.
   */
  @ManyToOne(targetEntity = TypAkordu.class, fetch = FetchType.EAGER)
  @JoinColumn(name = "typAkordu_id")
  public TypAkordu getTypAkordu() {
    return typAkordu;
  }

  /**
   * Ulozi typ akordu.
   * @param typAkordu - typ akordu.
   */
  public void setTypAkordu(TypAkordu typAkordu) {
    this.typAkordu = typAkordu;
  }

  /**
   * Informace o intervalu.
   */
  @Override
  public String toString() {
    StringBuffer info = new StringBuffer(100);
    info.append("[ID='");
    info.append(id);
    info.append("',INTERVAL=");
    info.append(interval);
    info.append(",TYP-AKORDU=");
    info.append(typAkordu);
    info.append("]");
    return info.toString();
  }
}
