package cz.cr.sara.kytara.domain;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.GenericGenerator;

import cz.cr.sara.kytara.view.AkordView;

@Entity
@Table(name = "pisen")
@NamedQueries({ @NamedQuery(name = "pisen.findById", query = "FROM Pisen p WHERE p.id = :id"),
    @NamedQuery(name = "allPisne", query = "FROM Pisen p") })
public class Pisen
    implements Serializable {

  private static final long serialVersionUID = 1L;

  private String id; // id pisne
  private String nazev; // nazev pisne
  private String kapela; // kapela
  private String obsah; // text + akordy
  private Set<AkordView> akordy; // akordy v pisni info

  /**
   * @return Vrati id pisne.
   */
  @Id
  @GeneratedValue(generator = "system-uuid")
  @GenericGenerator(name = "system-uuid", strategy = "uuid2")
  @Column(name = "id", nullable = false)
  public String getId() {
    return id;
  }

  /**
   * Ulozi id pisne.
   * @param id - id pisne.
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * @return Vrati nazev pisne.
   */
  @Column(name = "nazev", nullable = false)
  public String getNazev() {
    return nazev;
  }

  /**
   * Ulozi nazev pisne.
   * @param nazev - nazev pisne.
   */
  public void setNazev(String nazev) {
    this.nazev = nazev;
  }

  /**
   * @return Vrati kapelu pisne.
   */
  @Column(name = "kapela", nullable = false)
  public String getKapela() {
    return kapela;
  }

  /**
   * Ulozi kapelu.
   * @param kapela - kapela pisne.
   */
  public void setKapela(String kapela) {
    this.kapela = kapela;
  }

  /**
   * @return Vrati obsah pisne.
   */
  @Column(name = "obsah", nullable = false)
  public String getObsah() {
    return obsah;
  }

  /**
   * Ulozi obsah pisne.
   * @param obsah - obsah pisne.
   */
  public void setObsah(String obsah) {
    this.obsah = obsah;
  }

  /**
   * @return Vrati akordy pisne.
   */
  @Transient
  public Set<AkordView> getAkordy() {
    return akordy;
  }

  /**
   * Ulozi akordy pisne.
   * @param akordy - akordy pisne.
   */
  public void setAkordy(Set<AkordView> akordy) {
    this.akordy = akordy;
  }

  /**
   * Informace o pisni.
   */
  @Override
  public String toString() {
    StringBuffer info = new StringBuffer(100);
    info.append("[ID='");
    info.append(id);
    info.append("',NAZEV=");
    info.append(nazev);
    info.append(",KAPELA=");
    info.append(kapela);
    info.append(",OBSAH=");
    info.append(obsah);
    info.append(",AKORDY=");
    info.append(akordy);
    info.append("]");
    return info.toString();
  }
}
