package cz.cr.sara.kytara.domain;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;

import org.hibernate.annotations.GenericGenerator;

/**
 * Poloha akordu.
 * @author lukas
 */
@Entity
@Table(name = "poloha")
@NamedQueries({ @NamedQuery(name = "Poloha.prazceByStrunaAndTony",
    query = "SELECT p.prazec FROM Poloha p WHERE (p.struna = :struna and p.ton IN (:tons))") })
public class Poloha
    implements Serializable {

  private static final long serialVersionUID = 1L;

  private String id; // ID polohy
  private int prazec; // Poradi prazce
  private int struna; // Poradi struny
  private Ton ton; // Informace o tonu

  /**
   * @return Vrati ID polohy.
   */
  @Id
  @GeneratedValue(generator = "system-uuid")
  @GenericGenerator(name = "system-uuid", strategy = "uuid2")
  @Column(name = "id", nullable = false)
  public String getId() {
    return id;
  }

  /**
   * Ulozi ID polohy.
   * @param id - ID polohy.
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * @return Vrati poradi prazce.
   */
  @Column(name = "prazec", nullable = false)
  public int getPrazec() {
    return prazec;
  }

  /**
   * Ulozi poradi prazce.
   * @param prazec - poradi prazce.
   */
  public void setPrazec(int prazec) {
    this.prazec = prazec;
  }

  /**
   * @return Vrati poradi struny.
   */
  @Column(name = "struna", nullable = false)
  public int getStruna() {
    return struna;
  }

  /**
   * Ulozi poradi struny.
   * @param struna - poradi struny.
   */
  public void setStruna(int struna) {
    this.struna = struna;
  }

  /**
   * @return Vrati informaci o tonu.
   */
  @ManyToOne(targetEntity = Ton.class, fetch = FetchType.EAGER)
  @JoinColumn(name = "ton_id")
  public Ton getTon() {
    return ton;
  }

  /**
   * Ulozi informaci o tonu.
   * @param ton - informace o tonu.
   */
  public void setTon(Ton ton) {
    this.ton = ton;
  }

  /**
   * Poloha akordu.
   */
  @Override
  public String toString() {
    StringBuffer info = new StringBuffer(100);
    info.append("[ID-POLOHY='");
    info.append(id);
    info.append("',PORADI-PRAZCE=");
    info.append(prazec);
    info.append(",PORADI-STRUNY=");
    info.append(struna);
    info.append(",TON=");
    info.append(ton);
    info.append("]");
    return info.toString();
  }
}
