package cz.cr.sara.kytara.domain;

import java.io.Serializable;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.OrderBy;

/**
 * Informace o jednom tonu.
 * @author lukas
 */
@Entity
@Table(name = "ton")
@NamedQueries({ @NamedQuery(name = "Ton.tonById", query = "FROM Ton t WHERE t.id = :id"),
    @NamedQuery(name = "Ton.idTonByNazev", query = "SELECT t.id FROM Ton t WHERE t.nazev = :nazev"),
    @NamedQuery(name = "Ton.allNazev", query = "SELECT t.nazev FROM Ton t ORDER BY t.id ASC"),
    @NamedQuery(name = "Ton.all", query = "FROM Ton t ORDER BY t.id") })
public class Ton
    implements Serializable {
  private static final long serialVersionUID = 1L;

  private int id; // ID tonu
  private String nazev; // Nazev tonu
  private List<Poloha> polohy; // Seznam poloh

  /**
   * @return Vrati ID tonu.
   */
  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Column(name = "id", nullable = false)
  public int getId() {
    return id;
  }

  /**
   * Ulozi ID tonu.
   * @param id - ID tonu.
   */
  public void setId(int id) {
    this.id = id;
  }

  /**
   * @return Vrati nazev tonu.
   */
  @Column(name = "nazev", nullable = false)
  public String getNazev() {
    return nazev;
  }

  /**
   * Ulozi nazev tonu.
   * @param nazev - nazev tonu.
   */
  public void setNazev(String nazev) {
    this.nazev = nazev;
  }

  /**
   * @return Vrati seznam poloh.
   */
  @OneToMany(mappedBy = "ton", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
  @OrderBy(clause = ("struna ASC"))
  public List<Poloha> getPolohy() {
    return polohy;
  }

  /**
   * Ulozi seznam poloh.
   * @param polohy - seznam poloh.
   */
  public void setPolohy(List<Poloha> polohy) {
    this.polohy = polohy;
  }

  /**
   * Informace o jednom tonu.
   */
  @Override
  public String toString() {
    StringBuffer info = new StringBuffer(100);
    info.append("[ID=");
    info.append(id);
    info.append(",NAZEV='");
    info.append(nazev);
    info.append("',POCET-POLOH=");
    info.append((polohy != null)
        ? polohy.size()
        : 0);
    info.append("]");
    return info.toString();
  }
}
