package cz.cr.sara.kytara.repository;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;

/**
 * Spousteni definovaneho zpracovani v DBS v transakci.
 * @author lukas
 */
public abstract class TransactionCallback {
  protected EntityManager em;

  /**
   * Spusteni definovaneho zpracovani v DBS v transakci.
   * @param emf - tovarna entit pro praci s DBS.
   * @return Vrati vyhledana nebo informacni data.
   * @throws RuntimeException
   */
  public Object run(EntityManagerFactory emf) throws RuntimeException {
    if (emf == null) {
      throw new NullPointerException("Chybne vstupni parametry");
    }

    em = emf.createEntityManager();
    EntityTransaction tx = null;
    try {
      tx = em.getTransaction();
      tx.begin();

      Object o = runInTransaction();

      tx.commit();
      return o;
    }
    catch (Exception ex) {
      if (tx != null && tx.isActive()) {
        tx.rollback();
      }
      throw new RuntimeException(ex);
    }
    finally {
      em.close();
    }
  }

  /**
   * Definovane zpracovani v DBS.
   * @return Vrati vyhledana nebo informacni data.
   */
  public abstract Object runInTransaction();
}
