package cz.cr.sara.kytara.repository.impl;

import java.util.List;

import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceUnit;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.cr.sara.kytara.domain.Pisen;
import cz.cr.sara.kytara.repository.PisenDAO;
import cz.cr.sara.kytara.repository.TransactionCallback;

@Repository("pisenDao")
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
public class PisenDAOImpl
    implements PisenDAO {

  @PersistenceUnit(unitName = "jpaEntityFactory")
  private EntityManagerFactory emf;

  @Override
  public Pisen getPisenById(final String id) {
    Object data = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        Pisen pisen = em.createNamedQuery("pisen.findById",
            Pisen.class).setParameter("id",
            id).getSingleResult();
        return pisen;
      }
    }.run(emf);
    Pisen pisen = (data != null)
        ? (Pisen) data
        : null;
    if (pisen == null) {
      throw new EntityNotFoundException("Nebyla vyhledana pisen podle id pisne");
    }
    return pisen;
  }

  @Override
  public List<Pisen> getAllPisen() {
    Object data = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        List<Pisen> pisne = em.createNamedQuery("allPisne",
            Pisen.class).getResultList();
        return pisne;
      }
    }.run(emf);
    @SuppressWarnings("unchecked")
    List<Pisen> pisne = (data != null)
        ? (List<Pisen>) data
        : null;
    if (pisne == null || pisne.size() == 0) {
      throw new EntityNotFoundException("Nebyly vyhledany zadne pisne");
    }
    return pisne;
  }
}
