package cz.cr.sara.kytara.repository.impl;

import java.util.List;

import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.repository.PolohaDAO;
import cz.cr.sara.kytara.repository.TransactionCallback;

/**
 * Vyhledavani poloh prazcu v DBS.
 * @author lukas
 */
@Repository("polohaDao")
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
public class PolohaDAOImpl
    implements PolohaDAO {

  private static final Logger log = LoggerFactory.getLogger(PolohaDAOImpl.class);

  @PersistenceUnit(unitName = "jpaEntityFactory")
  private EntityManagerFactory emf;

  /**
   * Vyhleda a vrati poradi prazcu.
   * @param struna - poradi struny.
   * @param tony - seznamm tonu.
   * @ret Vrati poradi prazcu.
   */
  @Override
  public List<Integer> getPrazceByStrunaAndTony(final int struna,
      final List<Ton> tony) {
    Object data = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        List<Integer> prazce = em.createNamedQuery("Poloha.prazceByStrunaAndTony",
            Integer.class).setParameter("struna",
            struna).setParameter("tons",
            tony).getResultList();
        return prazce;
      }
    }.run(emf);
    @SuppressWarnings("unchecked")
    List<Integer> prazce = (data != null)
        ? (List<Integer>) data
        : null;
    if (prazce == null || prazce.size() == 0) {
      throw new EntityNotFoundException("Nebyly vyhledany poradi zadnych prazcu");
    }
    if (log.isDebugEnabled()) {
      log.debug("Vyhledane poradi prazcu: " + prazce);
    }
    return prazce;
  }
}
