package cz.cr.sara.kytara.repository.impl;

import java.util.List;

import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceUnit;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.repository.TonDAO;
import cz.cr.sara.kytara.repository.TransactionCallback;

/**
 * Vyhledavani tonu v DBS.
 * @author lukas
 */
@Repository("TonDao")
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
public class TonDAOImpl
    implements TonDAO {

  @PersistenceUnit(unitName = "jpaEntityFactory")
  private EntityManagerFactory emf;

  /**
   * Vrati ton pro pozadovane ID.
   */
  @Override
  public Ton getTonById(final int id) {
    Object ton = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        Ton ton = em.createNamedQuery("Ton.tonById",
            Ton.class).setParameter("id",
            id).getSingleResult();
        return ton;
      }
    }.run(emf);
    if (ton == null) {
      throw new EntityNotFoundException("Nebyl vyhledan ton pro ID: " + id);
    }
    return (Ton) ton;
  }

  /**
   * @return Vrati nazvy vsech tonu serazenych podle id.
   */
  @Override
  public List<String> getAllNazev() {
    Object data = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        List<String> nazvyTonu = em.createNamedQuery("Ton.allNazev",
            String.class).getResultList();
        return nazvyTonu;
      }
    }.run(emf);
    @SuppressWarnings("unchecked")
    List<String> nazvyTonu = (data != null)
        ? (List<String>) data
        : null;
    if (nazvyTonu == null || nazvyTonu.size() == 0) {
      throw new EntityNotFoundException("Nebyly vyhledany nazvy vsech tonu");
    }
    return nazvyTonu;
  }

  /**
   * Vyhleda a vraci ID tonu podle jeho nazvu.
   * @param nazevTon - nazev tonu.
   * @param emf - tovarna entit pro praci s DBS.
   * @return Vraci ID tonu podle jeho nazvu.
   */
  @Override
  public int getTonIdByTonNazev(final String nazevTon) {
    Object tonId = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        return em.createNamedQuery("Ton.idTonByNazev",
            Integer.class).setParameter("nazev",
            nazevTon).getSingleResult();
      }
    }.run(emf);
    if (tonId == null) {
      throw new EntityNotFoundException("Nebyl vyhledan ID tonu podle jeho nazvu: " + nazevTon);
    }
    return (int) tonId;
  }

  @Override
  public Ton getAllPolohaByTonId(final int tonId) {
    Object data = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        Ton ton = em.createNamedQuery("Ton.tonById",
            Ton.class).setParameter("id",
            tonId).getSingleResult();
        return ton;
      }
    }.run(emf);
    Ton ton = (data != null)
        ? (Ton) data
        : null;
    if (ton == null) {
      throw new EntityNotFoundException("Nebyl vyhledan ton");
    }
    return ton;
  }

  @Override
  public List<Ton> getAllTony() {
    Object data = new TransactionCallback() {
      @Override
      public Object runInTransaction() {
        List<Ton> tony = em.createNamedQuery("Ton.all",
            Ton.class).getResultList();
        return tony;
      }
    }.run(emf);
    @SuppressWarnings("unchecked")
    List<Ton> tony = (data != null)
        ? (List<Ton>) data
        : null;
    if (tony == null) {
      throw new EntityNotFoundException("Nebyl vyhledan zadny ton");
    }
    return tony;
  }
}
