package cz.cr.sara.kytara.repository.logic.impl;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

/**
 * Testovani a vytvareni spravnych hodnot pro tvorbu akordu.
 * @author lukas
 */
@Component("logicHelper")
public class RepositoryLogicHelper {
  /** Maximalni vzdalenost od zakladniho tonu */
  private static final int MAX_INTERVAL_VALUE = 11;
  /** Maximalni ID tonu */
  private static final int MAX_TONES_ID = 12;

  private final static Logger log = LoggerFactory.getLogger(RepositoryLogicHelper.class);

  /**
   * Vytvori a vrati spravny seznam vzdalenosti od zakladniho tonu.
   * @param intervaly - seznam vzdalenosti od zakladniho tonu.
   * @return Vrati spravny seznam vzdalenosti od zakladniho tonu.
   */
  public List<Integer> validateIntervals(List<Integer> intervaly) {
    List<Integer> validatedIntervaly = new ArrayList<>(intervaly.size());
    for (Integer interval : intervaly) {
      if (interval > RepositoryLogicHelper.MAX_INTERVAL_VALUE) {
        interval = interval - RepositoryLogicHelper.MAX_TONES_ID;
      }
      validatedIntervaly.add(interval);
    }
    return validatedIntervaly;
  }

  /**
   * Vytvori a vrati spravny seznam ID tonu.
   * @param zakladniTonId - ID zakladniho tonu.
   * @param intervaly - vzdalenost od zakladniho tonu.
   * @return Vrati spravny seznam ID tonu.
   */
  public List<Integer> validateTonIds(int zakladniTonId,
      List<Integer> intervaly) {
    if (log.isDebugEnabled()) {
      log.debug("ID zakladniho tonu: {}.",
          zakladniTonId);
      log.debug("Intervaly pred zmenou: {}.",
          intervaly);
    }

    List<Integer> tonIds = new ArrayList<>();
    tonIds.add(zakladniTonId);
    for (Integer interval : intervaly) {
      int tonId = zakladniTonId + interval;
      if (tonId > RepositoryLogicHelper.MAX_TONES_ID) {
        tonId = tonId - RepositoryLogicHelper.MAX_TONES_ID;
      }
      tonIds.add(tonId);
    }
    if (log.isDebugEnabled()) {
      log.debug("Spravny seznam ID tonu: {}.",
          tonIds);
    }
    return tonIds;
  }
}
