package cz.cr.sara.kytara.repository.logic.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Poskytuje varianty k nalezeni spravneho prazce.
 * @author lukas
 * 
 */
public class ZapisPrazcePriority {
  private static final Logger log = LoggerFactory.getLogger(ZapisPrazcePriority.class);
  public static final int NENALEZENY_PRAZEC = -1;

  /**
   * Urci, zda je nektery z prazcu stejny jako prazec zakladni.
   * @param prazce - mozne prazce.
   * @param zakladniPrazec - zakladni ton prazec.
   * @return Vraci cislo nalezeneho prazce nebo -1.
   */
  public static int isZakladniPrazec(Integer[] prazce,
      int zakladniPrazec) {
    int resultPrazec = NENALEZENY_PRAZEC;
    for (Integer prazec : prazce) {
      if (prazec == zakladniPrazec) {
        if (log.isDebugEnabled()) {
          log.debug("prazec id: {}.",
              prazec);
        }
        resultPrazec = prazec;
      }
    }
    return resultPrazec;
  }

  /**
   * Urci, zda je nektery z prazcu vzdalen od zakladniho prazce o danou vzdalenost.
   * @param prazce - mozne prazce.
   * @param zakladniPrazec - zakladni ton prazec.
   * @param vzdalenost - vzdalenost od zakladniho prazce.
   * @return Vraci cislo nalezeneho prazce nebo -1.
   */
  public static int isVzdalenODanouVzdalenost(Integer[] prazce,
      int zakladniPrazec,
      int vzdalenost) {
    int resultPrazec = NENALEZENY_PRAZEC;
    for (Integer prazec : prazce) {
      if (Math.abs(prazec - zakladniPrazec) == vzdalenost || Math.abs(zakladniPrazec - prazec) == vzdalenost) {
        resultPrazec = prazec;
      }
    }
    return resultPrazec;
  }
}
