package cz.cr.sara.kytara.service;

import java.util.List;
import java.util.Set;

import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.domain.TypAkordu;
import cz.cr.sara.kytara.service.pojos.Akord;
import cz.cr.sara.kytara.service.pojos.AkordInfo;
import cz.cr.sara.kytara.service.pojos.AkordPisen;

/**
 * Manazer pro vyhledavani akordu.
 * @author lukas
 */
public interface AkordManager {
  /**
   * Vyhleda a vrati akord podle typu nazvu akordu.
   * @param typAkordu - typ akordu.
   * @param nazevAkordu - nazev akordu.
   * @return Vrati akord podle typu.
   */
  public Akord getAkordByTypAndNazev(String typAkordu,
      String nazevAkordu);

  /**
   * Vyhleda a vrati vsechny typy akordu.
   * @return Vrati vsechny typy akordu.
   */
  public List<TypAkordu> getAllTypAkordu();

  /**
   * Vyhleda a vrati vsechny typy triad.
   * @return Vrati vsechny typy triad.
   */
  public List<TypAkordu> getAllTriady();

  /**
   * Vyhleda a vrati triadu podle typu a nazvu akordu.
   * @param typAkordu - typ akordu.
   * @param nazevAkordu - nazev akordu.
   * @return Vrati triadu podle typu akordu.
   */
  public Akord getTriadaByTypAndNazev(String typTriady,
      String nazev);

  /**
   * Vyhleda a vrati nazvy vsech tonu.
   * @return Vrati nazvy vsech tonu.
   */
  public List<String> getAllNazev();

  /**
   * Vyhleda a vrati vsechny tony.
   * @return Vrati vsechny tony.
   */
  public List<Ton> getAllTony();
  
  /**
   * Vyhleda a vrati seznam akorduInfo podle tonyId.
   * @return Vrati seznam akorduInfo podle tonyId.
   */
  public List<AkordInfo> getAkordyInfoByTonyId(List<Integer> tonyId);
  
  /**
   * Vyhleda a vrati akordy podle akordu v pisni.
   * @param akordy - list akordu v pisni.
   * @return Vrati akordy podle nazvu a typu akordu.
   */
  public Set<Akord> getAkordyByAkordyPisen(List<AkordPisen> akordyPisen);
}
