package cz.cr.sara.kytara.service.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.domain.TypAkordu;
import cz.cr.sara.kytara.repository.AkordDAO;
import cz.cr.sara.kytara.repository.TonDAO;
import cz.cr.sara.kytara.service.AkordManager;
import cz.cr.sara.kytara.service.logic.BuildAkordInfo;
import cz.cr.sara.kytara.service.pojos.Akord;
import cz.cr.sara.kytara.service.pojos.AkordInfo;
import cz.cr.sara.kytara.service.pojos.AkordPisen;
import cz.cr.sara.kytara.service.pojos.BuildAkordu;

/**
 * Manazer pro vyhledavani akordu.
 * @author lukas
 */
@Service("akordManager")
public class AkordManagerImpl
    implements AkordManager {
  private static final Logger log = LoggerFactory.getLogger(AkordManager.class);

  @Autowired
  AkordDAO akordDao;

  @Autowired
  TonDAO tonDao;

  @Autowired
  BuildAkordInfo buildAkordInfo;

  /**
   * Vyhleda a vrati akord podle typu.
   * @param typAkordu - typ akordu.
   * @param nazevAkordu - nazev akordu.
   * @return Vrati akord podle typu.
   */
  @Override
  public Akord getAkordByTypAndNazev(String typAkordu,
      String nazevAkordu) {
    return akordDao.getAkordByTypAkorduAndNazev(typAkordu,
        nazevAkordu);
  }

  /**
   * Vyhleda a vrati nazvy vsech tonu.
   * @return Vrati nazvy vsech tonu.
   */
  @Override
  public List<String> getAllNazev() {
    return tonDao.getAllNazev();
  }

  /**
   * Vyhleda a vrati vsechny typy akordu.
   * @return Vrati vsechny typy akordu.
   */
  public List<TypAkordu> getAllTypAkordu() {
    return akordDao.getAllTypAkordu();
  }

  @Override
  public List<TypAkordu> getAllTriady() {
    return akordDao.getAllTriads();
  }

  @Override
  public Akord getTriadaByTypAndNazev(String typTriady,
      String nazevAkordu) {
    return akordDao.getTriadaByTypAkordu(typTriady,
        nazevAkordu);
  }

  @Override
  public List<Ton> getAllTony() {
    return tonDao.getAllTony();
  }

  @Override
  public List<AkordInfo> getAkordyInfoByTonyId(List<Integer> tonyId) {
    if (log.isDebugEnabled()) {
      log.debug("tonyId {}.",
          tonyId);
    }
    List<AkordInfo> akordy = new ArrayList<AkordInfo>();
    List<BuildAkordu> buildAkordy = buildAkordInfo.getIntervalyByTonyId(tonyId);
    for (BuildAkordu buildAkord : buildAkordy) {
      List<TypAkordu> typyAkordu = akordDao.getTypyAkorduByIntervaly(buildAkord.getIntervaly());
      for (TypAkordu typAkordu : typyAkordu) {
        AkordInfo akordInfo = new AkordInfo();
        akordInfo.setTon(buildAkordInfo.getTonNazevByTonId(buildAkord.getZakladniTonId()));
        akordInfo.setTypAkordu(typAkordu.getTyp());
        akordy.add(akordInfo);
      }
    }
    return akordy;
  }

  @Override
  public Set<Akord> getAkordyByAkordyPisen(List<AkordPisen> akordyPisen) {
    Set<Akord> result = new HashSet<Akord>();
    for (AkordPisen akordPisen : akordyPisen) {
      Akord akord = getAkordByTypAndNazev(akordPisen.getTyp(),
          akordPisen.getNazev());
      if (log.isDebugEnabled()) {
        log.debug("akord: {}.",
            akord);
      }
      result.add(akord);
    }
    return result;
  }
}
