package cz.cr.sara.kytara.service.logic.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cz.cr.sara.kytara.service.pojos.AkordPisen;

public class AkordPisenMapperImpl {
  private static final Logger log = LoggerFactory.getLogger(AkordPisenMapperImpl.class);
  private final String ODDELOVAC_TONU = "/";
  private final int DELKA_AKORDU_DVA_ZNAKY = 2;
  private final int DELKA_AKORDU_JEDEN_ZNAK = 1;
  private final String AKORD_DUR = "dur";
  private final Set<String> TONY_NAZEV;
  private final Set<String> TYPY_AKORDU;
  private Map<String, String> nazevAkordDbVersusPisen = new HashMap<String, String>();
  public static final String NAZEV_AKORDU_ODDELENI = " ";

  public AkordPisenMapperImpl(Set<String> tonyNazev,
      Set<String> typyAkordu) {
    this.TONY_NAZEV = tonyNazev;
    this.TYPY_AKORDU = typyAkordu;
  }

  /**
   * Rozcleni akord na nazev a typ a vraci mapu akordu v pisni.
   * @param akordyVPisni - akordy v pisni.
   * @return Vraci mapu akordu v pisni.
   */
  public List<AkordPisen> getNazevATypAkordu(Set<String> akordyVPisni) {
    Map<String, String> map = getMapTonyMeziDbAPisni();
    List<AkordPisen> result = new ArrayList<AkordPisen>();
    for (String akordPisen : akordyVPisni) {
      if (log.isDebugEnabled()) {
        log.debug("akordPisen: {}.",
            akordPisen);
      }
      AkordPisen ap = getAkordPisen(map,
          akordPisen,
          DELKA_AKORDU_DVA_ZNAKY);
      if (ap == null || ap.getNazev() == null) {
        ap = getAkordPisen(map,
            akordPisen,
            DELKA_AKORDU_JEDEN_ZNAK);
      }
      result.add(ap);
    }
    return result;
  }

  /**
   * Vyhleda a vrati informace o akordu v pisni.
   * @param map - mapa nazvu tonu v pisni (klic) versus v db.
   * @param akordPisen - akord v pisni.
   * @param delkaNazvu - jak dlouhy akord se hleda.
   * @return Vrati informace o akordu v pisni.
   */
  private AkordPisen getAkordPisen(Map<String, String> map,
      String akordPisen,
      int delkaNazvu) {
    AkordPisen ap = new AkordPisen();
    for (String ton : map.keySet()) {
      if (akordPisen.length() >= delkaNazvu && ton.equals(akordPisen.substring(0,
          delkaNazvu))) {
        String typAkordu = getAkordTyp(akordPisen.substring(delkaNazvu));
        ap.setNazev(map.get(ton));
        ap.setTyp(typAkordu);
        vlozDoMapyNazvyAkordu(map.get(ton), typAkordu, akordPisen);
      }
    }
    return ap;
  }
  
  /**
   * Vlozi zaznam do mapy spravujici nazvy akordu v db (klic) a v pisni.
   * @param ton - ton v db.
   * @param typAkordu - typ akordu v db.
   * @param akordPisen - cely akord v pisni.
   */
  private void vlozDoMapyNazvyAkordu(String ton, String typAkordu, String akordPisen){
    nazevAkordDbVersusPisen.put(ton + NAZEV_AKORDU_ODDELENI + typAkordu,
        akordPisen);
  }

  /**
   * Vytvori a vrati mapu nazvu tonu v pisni (klic) versus v db.
   * @return Vrati mapu nazvu tonu v pisni versus v db.
   */
  private Map<String, String> getMapTonyMeziDbAPisni() {
    Map<String, String> map = new HashMap<String, String>();
    for (String ton : this.TONY_NAZEV) {
      String[] tony_ = (ton.split(ODDELOVAC_TONU));
      for (String t : tony_) {
        map.put(t,
            ton);
      }
    }
    return map;
  }

  /**
   * Zvaliduje a vrati typ akordu.
   * @param typ - typ akordu z pisne.
   * @return Vrati typ akordu.
   */
  private String getAkordTyp(String typ) {
    if (log.isDebugEnabled()) {
      log.debug("typ: {}.",
          typ);
    }
    if (typ == null || typ == "") {
      typ = AKORD_DUR;
    }
    for (String typDb : this.TYPY_AKORDU) {
      if (typ.equals(typDb)) {
        return typDb;
      }
    }
    return AKORD_DUR;
  }

  public Map<String, String> getNazevAkordDbVersusPisen() {
    return nazevAkordDbVersusPisen;
  }

  /**
   * Vraci mapovani nazvu akordu mezi db a pisni.
   * @return
   */
  public void setNazevAkordDbVersusPisen(Map<String, String> nazevAkordDbVersusPisen) {
    this.nazevAkordDbVersusPisen = nazevAkordDbVersusPisen;
  }
}
