package cz.cr.sara.kytara.view;

import java.util.ArrayList;
import java.util.List;

import cz.cr.sara.kytara.ciselniky.VyberAkordu;
import cz.cr.sara.kytara.domain.Poloha;
import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.service.pojos.Akord;
import cz.cr.sara.kytara.service.pojos.PolohaAkordu;

/**
 * Informace o jednom akordu.
 * @author lukas
 */
public class AkordView {
  private String nazev; // Nazev akordu
  private final String ton; // Zakladni ton
  private final String typAkordu; // Typ akordu

  private final List<String> tony; // Seznam tonu
  private final List<AkordView.Hmat> hmaty; // Seznam hmatu
  private final VyberAkordu vyberAkordu; // Vyber akordu

  /**
   * Informace o jednom akordu.
   * @param akord - jeden akord.
   * @param vyberAkordu - vyber akordu.
   * @throws NullPointerException
   */
  public AkordView(Akord akord,
      VyberAkordu vyberAkordu) throws NullPointerException {
    try {
      if (akord == null || vyberAkordu == null) {
        throw new NullPointerException("Chybne vstupni parametry");
      }

      // Vyber akordu
      this.vyberAkordu = vyberAkordu;

      // Nazev akordu
      this.nazev = akord.getNazev();
      int pos = this.nazev.indexOf(" ");
      if (pos <= 0) {
        throw new NullPointerException("Chybny nazev akordu: " + this.nazev);
      }
      // Ton
      this.ton = this.nazev.substring(0,
          pos).trim();
      // Typ akordu
      this.typAkordu = this.nazev.substring(pos + 1).trim();

      // Seznam tonu
      List<Ton> tony = akord.getTony();
      this.tony = new ArrayList<>(tony.size());
      for (Ton ton : tony) {
        this.tony.add(ton.getNazev());
      }

      // Seznam hmatu
      List<PolohaAkordu> polohyAkordu = akord.getPolohyAkordu();
      this.hmaty = new ArrayList<>(polohyAkordu.size());
      int poradi = 0;
      for (PolohaAkordu polohaAkordu : polohyAkordu) {
        List<Poloha> seznamPoloha = polohaAkordu.getPoloha();
        if (seznamPoloha == null || seznamPoloha.size() != AkordView.Hmat.POCET_STRUN) {
          StringBuffer error = new StringBuffer(100);
          error.append("Chybny pocet strun v poloze akordu: ");
          if (seznamPoloha != null) {
            error.append(seznamPoloha.size());
          }
          else {
            error.append("null");
          }
          throw new NullPointerException(error.toString());
        }
        StringBuffer prazce = new StringBuffer(AkordView.Hmat.POCET_STRUN);
        boolean first = true;
        for (Poloha poloha : seznamPoloha) {
          if (!first) {
            prazce.append(AkordView.Hmat.ODDELOVAC_STRUN);
          }
          prazce.append(poloha.getPrazec());
          first = false;
        }
        AkordView.Hmat hmat = new AkordView.Hmat(++poradi,
            prazce.toString());
        this.hmaty.add(hmat);
      }
    }
    catch (Exception e) {
      throw new NullPointerException("AkordView.AkordView: " + e.getMessage());
    }
  }

  /**
   * @return Vrati nazev akordu.
   */
  public String getNazev() {
    return nazev;
  }

  /**
   * Nastavuje nazev akordu.
   * @param nazev - nazev akordu.
   */
  public void setNazev(String nazev) {
    this.nazev = nazev;
  }

  /**
   * @return Vrati ton.
   */
  public String getTon() {
    return ton;
  }

  /**
   * @return Vrati typ akordu.
   */
  public String getTypAkordu() {
    return typAkordu;
  }

  /**
   * @return Vrati seznam tonu.
   */
  public List<String> getTony() {
    return tony;
  }

  /**
   * @return Vrati seznam hmatu.
   */
  public List<AkordView.Hmat> getHmaty() {
    return hmaty;
  }

  /**
   * @return Vrati vyber akordu.
   */
  public VyberAkordu getVyberAkordu() {
    return vyberAkordu;
  }

  /**
   * Informace o jednom akordu.
   */
  @Override
  public String toString() {
    StringBuffer info = new StringBuffer(100);
    info.append("[NAYEV-AKORDU='");
    info.append(nazev);
    info.append("',TON='");
    info.append(ton);
    info.append("',TYP-AKORDU='");
    info.append(typAkordu);
    info.append("',SEZNAM-TONU=");
    info.append(tony);
    info.append(",SEZNAM-HMATU=");
    info.append(hmaty);
    info.append(",VYBER-AKORDU=");
    info.append(vyberAkordu);
    info.append("]");
    return info.toString();
  }

  /**
   * Informace o jednom hmatu.
   * @author lukas
   */
  public static class Hmat {
    /** Pocet strun */
    public static final int POCET_STRUN = 6;
    /** Oddelovac poradi prazcu pro jednotlive struny */
    public static final String ODDELOVAC_STRUN = "|";

    private final int poradi; // Poradi hmatu
    private final String prazce; // Poradi prazcu pro jednotlive struny

    /**
     * Informace o jednom hmatu.
     * @param poradi - poradi hmatu.
     * @param prazce - poradi prazcu pro jednotlive struny.
     * @throws NullPointerException
     */
    public Hmat(int poradi,
        String prazce) throws NullPointerException {
      try {
        if (poradi <= 0 || prazce == null || prazce.length() == 0 || prazce.split("[|]").length != AkordView.Hmat.POCET_STRUN) {
          StringBuffer error = new StringBuffer(100);
          error.append("Chybne vstupni parametry: poradi=");
          error.append(poradi);
          error.append(" , prazce='");
          error.append(prazce);
          error.append("'");
          throw new NullPointerException(error.toString());
        }

        this.poradi = poradi;
        this.prazce = prazce;
      }
      catch (Exception e) {
        throw new NullPointerException("AkordView.Hmat.Hmat: " + e.getMessage());
      }
    }

    /**
     * @return Vrati poradi hmatu.
     */
    public int getPoradi() {
      return poradi;
    }

    /**
     * @return Vrati poradi prazcu pro jednotlive struny.
     */
    public String getPrazce() {
      return prazce;
    }

    /**
     * Informace o jednom hmatu.
     */
    @Override
    public String toString() {
      StringBuffer info = new StringBuffer(100);
      info.append("[PORADI-HMATU=");
      info.append(poradi);
      info.append(",PORADI-PRAZCU='");
      info.append(prazce);
      info.append("']");
      return info.toString();
    }
  }
}
