package cz.cr.sara.kytara.view;

import java.util.ArrayList;
import java.util.List;

import cz.cr.sara.kytara.ciselniky.VyberAkordu;
import cz.cr.sara.kytara.domain.Poloha;
import cz.cr.sara.kytara.domain.Ton;

/**
 * Informace o jednom tonu.
 * @author lukas
 */
public class TonView {
  private final String ton; // Nazev tonu
  private final List<List<String>> nazvyTonu; // Seznam tonu
  private final AkordView.Hmat hmat; // Informace o hmatu

  /**
   * Informace o jednom tonu.
   * @param ton - jeden ton.
   * @param nazvyTonu - nazvy vsech tonu.
   * @throws NullPointerException
   */
  public TonView(Ton ton,
      List<String> nazvyTonu) throws NullPointerException {
    try {
      if (ton == null || nazvyTonu == null) {
        throw new NullPointerException("Chybne vstupni parametry");
      }

      // Seznam nazvu tonu
      this.nazvyTonu = new ArrayList<List<String>>();
      List<String> lineTony = new ArrayList<String>(VyberAkordu.COUNT_TONY_LINE);
      for (String tonNazev : nazvyTonu) {
        lineTony.add(tonNazev);
        if (lineTony.size() == VyberAkordu.COUNT_TONY_LINE) {
          this.nazvyTonu.add(lineTony);
          lineTony = new ArrayList<String>(VyberAkordu.COUNT_TONY_LINE);
        }
      }
      if (lineTony.size() > 0) {
        this.nazvyTonu.add(lineTony);
      }

      // Nazev tonu
      this.ton = ton.getNazev();

      // Hmat
      List<Poloha> seznamPoloha = ton.getPolohy();
      if (seznamPoloha == null || seznamPoloha.size() != AkordView.Hmat.POCET_STRUN) {
        throw new NullPointerException("Chybny pocet strun v poloze akordu");
      }
      StringBuffer prazce = new StringBuffer(AkordView.Hmat.POCET_STRUN);
      boolean first = true;
      for (Poloha poloha : seznamPoloha) {
        if (!first) {
          prazce.append(AkordView.Hmat.ODDELOVAC_STRUN);
        }
        prazce.append(poloha.getPrazec());
        first = false;
      }
      this.hmat = new AkordView.Hmat(1,
          prazce.toString());
    }
    catch (Exception e) {
      throw new NullPointerException("TonView.TonView: " + e.getMessage());
    }
  }

  /**
   * @return Vrati nazev tonu.
   */
  public String getTon() {
    return ton;
  }

  /**
   * @return Vrati nazvy tonu.
   */

  public List<List<String>> getNazvyTonu() {
    return nazvyTonu;
  }

  /**
   * @return Vrati informace o hmatu.
   */
  public AkordView.Hmat getHmat() {
    return hmat;
  }

  /**
   * Informace o jednom tonu.
   */
  @Override
  public String toString() {
    StringBuffer info = new StringBuffer(100);
    info.append("[TON='");
    info.append(ton);
    info.append("',NAZVY_TONU=");
    info.append(nazvyTonu);
    info.append(",HMAT=");
    info.append(hmat);
    info.append("]");
    return info.toString();
  }

}
