package cz.cr.sara.kytara.web;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import cz.cr.sara.kytara.ciselniky.AkordVyber;
import cz.cr.sara.kytara.service.AkordManager;
import cz.cr.sara.kytara.service.pojos.Akord;
import cz.cr.sara.kytara.service.pojos.AkordInfo;
import cz.cr.sara.kytara.view.AkordView;
import cz.cr.sara.kytara.view.AkordView.Hmat;

/**
 * Kontroler pro akordy.
 * @author lukas
 */
@Controller
@RequestMapping("/akordy")
public class AkordFormController {
  /** Implicitne vyhledavany akord */
  private static final String[] AKORD = { "C",
      "dur" };

  @Autowired
  private AkordManager akordManager;

  private static final Logger log = LoggerFactory.getLogger(AkordFormController.class);

  /**
   * Vyhledavani a zobrazovani kytarovych akordu.
   */
  @RequestMapping(value = { "/akordy" }, method = { RequestMethod.POST,
      RequestMethod.GET })
  protected ModelAndView akordyFind(@ModelAttribute("akordInfo") AkordInfo akordInfo) {
    if (akordInfo.getTon() == null) {
      akordInfo.setTon(AkordFormController.AKORD[0]);
      akordInfo.setTypAkordu(AkordFormController.AKORD[1]);
    }
    // Vyber akordu
    AkordVyber akordVyber = AkordVyber.getInstance(akordManager);

    // Vyhledani akordu
    Akord akord = akordManager.getAkordByTypAndNazev(akordInfo.getTypAkordu(),
        akordInfo.getTon());

    // Informace o vyhledanem akordu
    AkordView akordView = new AkordView(akord,
        akordVyber.getVyberAkordu());

    if (log.isDebugEnabled()) {
      log.debug("Nazev akordu: {}.",
          akordView.getNazev());
      log.debug("Seznam tonu akordu: {}.",
          akordView.getTony());
      log.debug("Typ akordu: {}.",
          akordView.getTypAkordu());
      for (Hmat hmat : akordView.getHmaty()) {
        log.debug("Hmat: {}",
            hmat.getPrazce());
      }
    }
    return new ModelAndView("/akordy/akordy",
        "akord",
        akordView);
  }
}
