package cz.cr.sara.kytara.web;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import cz.cr.sara.kytara.ciselniky.AkordVyber;
import cz.cr.sara.kytara.domain.Ton;
import cz.cr.sara.kytara.service.AkordManager;
import cz.cr.sara.kytara.service.pojos.AkordInfo;
import cz.cr.sara.kytara.service.pojos.AllTony;

/**
 * Kontroler pro vyhledani akordu podle zadanych tonu.
 * @author lukas
 */
@Controller
@RequestMapping("/findAkord")
public class FindAkordFormController {

  @Autowired
  private AkordManager akordManager;

  private static final Logger log = LoggerFactory.getLogger(FindAkordFormController.class);

  /**
   * Vyhledavani a zobrazovani vsech tonu.
   */
  @RequestMapping(value = { "/tony" }, method = RequestMethod.GET)
  protected ModelAndView allTony(@ModelAttribute("vybraneTony") AllTony vybraneTony) {
    // Vyber akordu
    AkordVyber akordVyber = AkordVyber.getInstance(akordManager);

    // Vsechny tony nazvy
    List<Ton> tony = akordVyber.getVyberAkordu().getTony();

    if (log.isDebugEnabled()) {
      log.debug("Tony: {}.",
          tony);
    }
    return new ModelAndView("/findAkord/findAkord",
        "tony",
        tony);
  }

  /**
   * Najde akordy podle tonu.
   */
  @RequestMapping(value = { "/findAkordy" }, method = RequestMethod.POST, headers = "Accept=application/json")
  protected @ResponseBody
  List<AkordInfo> akordFind(@RequestBody String tony) {
    // Seznam ID-tonu
    String[] tony_ = tony.split("=")[1].split("A");
    List<Integer> idTony = new ArrayList<>(tony_.length);
    for (String id : tony_) {
      idTony.add(Integer.valueOf(id));
    }
    if (log.isDebugEnabled()) {
      log.debug("Tony: {}.",
          tony);
      log.debug("ID tonu: {}.",
          idTony);
    }
    List<AkordInfo> akordy = akordManager.getAkordyInfoByTonyId(idTony);
    Collections.sort(akordy);
    return akordy;
  }
}
