package cz.cr.sara.kytara.web;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import cz.cr.sara.kytara.ciselniky.TriadyVyber;
import cz.cr.sara.kytara.service.AkordManager;
import cz.cr.sara.kytara.service.pojos.Akord;
import cz.cr.sara.kytara.service.pojos.AkordInfo;
import cz.cr.sara.kytara.view.AkordView;
import cz.cr.sara.kytara.view.AkordView.Hmat;

/**
 * Kontroler pro triady.
 * @author lukas
 */
@Controller
@RequestMapping("/triady")
public class TriadyFormController {
  /** Implicitne vyhledavana triada */
  private static final String[] AKORD = { "C",
      "dur" };

  @Autowired
  private AkordManager akordManager;

  private static final Logger log = LoggerFactory.getLogger(TriadyFormController.class);

  /**
   * Vyhledavani a zobrazovani kytarovych triad.
   */
  @RequestMapping(value = { "/triady" }, method = { RequestMethod.POST,
      RequestMethod.GET })
  protected ModelAndView triadyFind(@ModelAttribute("akordInfo") AkordInfo akordInfo) {
    if (akordInfo.getTon() == null) {
      akordInfo.setTon(TriadyFormController.AKORD[0]);
      akordInfo.setTypAkordu(TriadyFormController.AKORD[1]);
    }
    // Vyber triad
    TriadyVyber triadyVyber = TriadyVyber.getInstance(akordManager);

    // Vyhledani triady
    Akord akord = akordManager.getTriadaByTypAndNazev(akordInfo.getTypAkordu(),
        akordInfo.getTon());

    // Informace o vyhledane triade
    AkordView akordView = new AkordView(akord,
        triadyVyber.getVyberAkordu());

    if (log.isDebugEnabled()) {
      log.debug("Nazev akordu: {}.",
          akordView.getNazev());
      log.debug("Seznam tonu akordu: {}.",
          akordView.getTony());
      log.debug("Typ akordu: {}.",
          akordView.getTypAkordu());
      for (Hmat hmat : akordView.getHmaty()) {
        log.debug("Hmat: {}",
            hmat.getPrazce());
      }
    }
    return new ModelAndView("/triady/triady",
        "akord",
        akordView);
  }
}
